@@ -7,7 +7,7 @@ from __future__ import print_function
     This uses the Python type P4API.P4Adapter, which is a wrapper for the
     Perforce ClientApi object.
     
-    $Id: //depot/r15.2/p4-python/P4.py#2 $
+    $Id: //depot/r16.2/p4-python/P4.py#1 $
     
     #*******************************************************************************
     # Copyright (c) 2007-2010, Perforce Software, Inc.  All rights reserved.
@@ -565,7 +565,17 @@ class P4(P4API.P4Adapter):
     def identify(cls):
         return P4API.identify()
     identify = classmethod(identify)
+
+    def log_messages(self):
+        for message in self.messages:
+            if message.severity == 3:
+                self.logger.error(message)
+            elif message.severity == 2:
+                self.logger.warning(message)
+            elif message.severity == 1:
+                self.logger.info(message)
     
+        
     def run(self, *args, **kargs):
         "Generic run method"
         context = {}
@@ -591,7 +601,14 @@ class P4(P4API.P4Adapter):
         if self.logger:
             self.logger.info("p4 " + " ".join(flatArgs))
         
-        result = P4API.P4Adapter.run(self, *flatArgs)
+        try:
+            result = P4API.P4Adapter.run(self, *flatArgs)
+        except P4Exception as e:
+            if self.logger:
+                self.log_messages()
+            raise e
+        if self.logger:
+            self.log_messages()
         
         if resultLogging and self.logger:
             self.logger.debug(result)
@@ -635,7 +652,13 @@ class P4(P4API.P4Adapter):
     
     def run_login(self, *args, **kargs):
         "Simple interface to make login easier"
-        self.input = self.password
+        
+        if "password" in kargs:
+            password = kargs["password"]
+            self.input = password
+            del kargs["password"]
+        else:
+            self.input = self.password
         return self.run("login", *args, **kargs)
     
     def run_password( self, oldpass, newpass, *args, **kargs ):
