@@ -25,7 +25,7 @@ ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
-$Id: //depot/r15.2/p4-python/PythonClientAPI.cpp#1 $
+$Id: //depot/r16.2/p4-python/PythonClientAPI.cpp#1 $
 *******************************************************************************/
  
 #include <Python.h>
@@ -965,6 +965,17 @@ PyObject * PythonClientAPI::FormatSpec( const char *type, PyObject * dict )
     Py_RETURN_NONE;
 }
 
+//
+// Sets the spec field in the cache, used for triggers with the new 16.1 ability to pass the spec definition
+//
+
+PyObject * PythonClientAPI::DefineSpec( const char *type, const char * spec)
+{
+    specMgr.AddSpecDef(type, spec);
+
+    Py_RETURN_TRUE;
+}
+
 //
 // Returns a dict whose keys contain the names of the fields in a spec of the
 // specified type. Not yet exposed to Python clients, but may be in future.
