@@ -23,7 +23,7 @@
  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
- $Id: //depot/r15.2/p4-python/SpecMgr.cpp#3 $
+ $Id: //depot/r16.2/p4-python/SpecMgr.cpp#1 $
  *******************************************************************************/
 
 /*******************************************************************************
@@ -54,6 +54,7 @@
 
 using namespace std;
 
+
 struct specdata {
     const char *type;
     const char *spec;
@@ -107,10 +108,12 @@ struct specdata {
 	"Stream;code:314;type:line;len:64;;"
 	"StreamAtChange;code:316;type:line;len:64;;"
 	"ServerID;code:315;type:line;ro;len:64;;"
-	"Type;code:318;type:select;len:10;val:writeable/readonly;;"
+	"Type;code:318;type:select;len:10;val:"
+	"writeable/readonly/graph/partitioned;;"
 	"Backup;code:319;type:select;len:10;val:enable/disable;;"
 	"View;code:311;type:wlist;words:2;len:64;;"
 	"ChangeView;code:317;type:llist;len:64;;"
+	"RepoView;code:320;type:llist;len:64;;"
     },
     {
 	"depot",
@@ -137,6 +140,7 @@ struct specdata {
 	"LdapConfig;code:410;type:line;len:128;;"
 	"LdapSearchQuery;code:411;type:line;len:128;;"
 	"LdapUserAttribute;code:412;type:line;len:128;;"
+	"LdapUserDNAttribute;code:414;type:line;len:128;;"
 	"Subgroups;code:404;type:wlist;len:32;opt:default;;"
 	"Owners;code:408;type:wlist;len:32;opt:default;;"
 	"Users;code:405;type:wlist;len:32;opt:default;;"
@@ -207,6 +211,8 @@ struct specdata {
     },
     {
 	"protect",
+	"SubPath;code:502;ro;len:64;;"
+	"Update;code:503;type:date;ro;fmt:L;len:20;;"
 	"Protections;code:501;fmt:C;type:wlist;words:5;opt:default;len:64;;"
     },
     {
@@ -216,7 +222,7 @@ struct specdata {
 	"Owner;code:853;fmt:R;len:32;;"
 	"RemoteUser;code:861;fmt:R;len:32;;"
 	"Options;code:854;type:line;len:32;val:"
-	"unlocked/lockednocompress/compress;;"
+	"unlocked/locked,nocompress/compress,copyrcs/nocopyrcs;;"
 	"Update;code:855;type:date;ro;fmt:L;len:20;;"
 	"Access;code:856;type:date;ro;fmt:L;len:20;;"
 	"Description;code:857;type:text;len:128;;"
@@ -235,6 +241,7 @@ struct specdata {
 	"Services;code:756;rq;len:128;;"
 	"Description;code:757;type:text;len:128;;"
 	"User;code:761;type:line;len:64;;"
+	"AllowedAddresses;code:763;type:wlist;len:64;;"
 	"ClientDataFilter;code:758;type:wlist;len:64;;"
 	"RevisionDataFilter;code:759;type:wlist;len:64;;"
 	"ArchiveDataFilter;code:760;type:wlist;len:64;;"
