@@ -24,7 +24,8 @@ import P4API
 import unittest, os, types, shutil, stat
 from subprocess import Popen, PIPE
 import sys
-import os
+import os, os.path
+import re
 
 def onRmTreeError( function, path, exc_info ):
     os.chmod( path, stat.S_IWRITE)
@@ -70,6 +71,17 @@ class TestP4Python(unittest.TestCase):
         if not os.path.isdir(directory):
             os.mkdir(directory)
 
+    def getServerPatchLevel(self, info):
+        c = re.compile("[^/]*/[^/]*/[^/]*/([^/]*)\s\(\d+/\d+/\d+\)")
+        
+        serverVersion = info[0]["serverVersion"]
+        m = c.match(serverVersion)
+        if m:
+            serverPatch = m.group(1)
+            return int(serverPatch)
+        else:
+            print("Cannot extract patch level from {0}".format(serverVersion))
+            sys.exit(1)
 
 class TestP4(TestP4Python):
 
@@ -278,6 +290,7 @@ class TestP4(TestP4Python):
             self.p4.run_login( )
         except P4.P4Exception:
             self.fail( "Failed to log on")
+        self.p4.run_login(password=password)
 
         try:
             self.p4.run_password( "", password)
@@ -546,6 +559,24 @@ class TestP4(TestP4Python):
         map.insert( '"//depot/dir with spaces/..." "//ws/dir with spaces/..."' )
         self.assertEqual( map.includes("//depot/dir with spaces/foo"), True, "Quotes not handled correctly" )
 
+        map.clear()
+        map = P4.Map(['//depot/a/... a/...', '+//depot/b/... b/...'])
+        self.assertEqual( map.as_array(), ['//depot/a/... a/...', '+//depot/b/... b/...'], "+ mappings not handled appropriatly" )
+        self.assertEqual( map.lhs(), ['//depot/a/...', '+//depot/b/...'], "+ mappings not handled appropriatly" )
+
+        # & map test disabled until fixed in P4API
+        map.clear()
+        map = P4.Map(['//depot/a/... a/...', '&//depot/a/... b/...'])
+        self.assertEqual( map.as_array(), ['//depot/a/... a/...', '&//depot/a/... b/...'], "& mappings not handled appropriatly" )
+        self.assertEqual( map.lhs(), ['//depot/a/...', '&//depot/a/...'], "& mappings not handled appropriatly" )
+
+        # test P4Map.translate and P4Map.translate_array
+        map.clear()
+        map = P4.Map(['//depot/a/... a/...', '&//depot/a/... b/...'])
+        self.assertEqual( map.translate("//depot/a/foo"), "a/foo", "P4Map.translate not handled correctly") 
+        self.assertEqual( map.translate("a/foo", 0), "//depot/a/foo", "P4Map.translate not handled correctly") 
+        self.assertEqual( map.translate_array("//depot/a/foo"), ["b/foo", "a/foo"], "P4Map.translate not handled correctly") 
+        
     def testThreads( self ):
             import threading
 
@@ -811,7 +842,7 @@ class TestP4(TestP4Python):
         """Check a single spec.
    Takes the name and a potential parameter as an argument.
    Some specs like change or triggers don't require (or are not even allowed) a parameter.
-   Other specs like depot or label require a parameter. Stteams are particularily picky.
+   Other specs like depot or label require a parameter. Streams are particularily picky.
 
    First, we pick up the spec definition from the server.
    Then we test whether we can set all fields without error. We need to distinguish
@@ -848,7 +879,10 @@ class TestP4(TestP4Python):
         spec_string = self.p4.format_spec(spec_name, trial_spec)
 
         p4_2 = P4.P4()
-        reparsed = p4_2.parse_spec(spec_name, spec_string)
+        try:
+            reparsed = p4_2.parse_spec(spec_name, spec_string)
+        except P4.P4Exception as e:
+            self.fail("Spec '{0}' failed with {1}".format(spec_name, e))    
         del p4_2
 
     def testAllSpecs( self ):
@@ -985,27 +1019,56 @@ class TestP4(TestP4Python):
         self.assertEqual(self.p4.p4enviro_file, TEST_P4ENVIRO, "Did not set P4ENVIRO correctly")
 
     def testLogger( self ):
+        import logging
+        try:
+            from StringIO import StringIO
+        except ImportError:
+            from io import StringIO
+        
         self.p4.connect()
 
-        d = { 'debugCalled' : False, 'infoCalled' : False }
-
-        class MockLogger:
-            def debug(self, txt):
-                d['debugCalled'] = True
-            def info(self, txt):
-                d['infoCalled'] = True
-
-        m = MockLogger()
-
-        self.p4.logger = m
-        self.assertEqual(self.p4.logger, m, "Logger not set correctly")
-        self.assertEqual(self.p4.debug, 0, "Debug has not 0")
+        # set up a String stream for log testing
+        stream = StringIO()
+        handler = logging.StreamHandler(stream)
+        format = logging.Formatter('%(levelname)s:%(message)s')
+        handler.formatter = format
+        
+        self.p4.logger = logging.getLogger('TestP4Logger')
+        self.p4.logger.setLevel(logging.INFO)
+        self.p4.logger.addHandler(handler)
+        
+        self.assertEqual(self.p4.debug, 0, "Debug is not 0")
 
+        # Simple log test with no debug output
         self.p4.run_info()
+        self.assertEqual(stream.getvalue(), "INFO:p4 info\n", "Logging stream contains '{0}'".format(stream.getvalue()))
+        
+        stream.truncate(0)
+        stream.seek(0) # not necessary in Python2, but required for Python3
+        
+        # Now with an exception, should raise a WARNING
+        
+        self.assertRaises(P4Exception, lambda : self.p4.run_files('//depot/foobar'))
+        
+        self.assertEqual(stream.getvalue(), 
+                         "INFO:p4 files //depot/foobar\n"
+                         "WARNING://depot/foobar - no such file(s).\n", 
+                         "Unexpected {0}".format(stream.getvalue()))
+        
+        stream.truncate(0)
+        stream.seek(0) # not necessary in Python2, but required for Python3
+       
+        # Now without an exception, but still with WARNING output - and now with DEBUG output as well
+        
+        self.p4.logger.setLevel(logging.DEBUG)
+        self.p4.run_files('//depot/foobar', exception_level=0)
 
-        self.assertTrue(d['debugCalled'], "Logger was not called with debug")
-        self.assertTrue(d['infoCalled'], "Logger was not called with info")
-
+        self.assertEqual(stream.getvalue(), 
+                         "INFO:p4 files //depot/foobar\n"
+                         "WARNING://depot/foobar - no such file(s).\n"
+                         "DEBUG:[]\n", 
+                         "Unexpected {0}".format(stream.getvalue()))
+        
         self.p4.logger = None
         self.assertEqual(self.p4.logger, None, "Logger not reset correctly")
 
@@ -1027,7 +1090,7 @@ class TestP4(TestP4Python):
                 os.chdir(current_pwd)
                 os.environ["PWD"] = current_pwd
             except Exception as e:
-                self.fail("P4.run_init() raised exception {}".format(e))
+                self.fail("P4.run_init() raised exception {0}".format(e))
 
     def testDVCS_clone( self ):
         self.p4.connect()
@@ -1067,8 +1130,87 @@ class TestP4(TestP4Python):
                 self.assertNotEqual(len(files),0, "No files found!")
                 dvcs.disconnect()
             except Exception as e:
-                self.fail("P4.clone() raised exception {}".format(e))
+                self.fail("P4.clone() raised exception {0}".format(e))
 
+    def testSpecdefTrigger( self ):
+        # need to create trigger
+        # save it in the depot
+        # link it from the trigger table (sys.executable)
+        # update the jobspec
+        # then try it out with a job
+        
+        self.assertTrue(os.path.exists("job_trigger.py"), "Can't find job_trigger.py")
+        with open("job_trigger.py") as f:
+            triggerCode = f.read()
+        
+        self.p4.connect()
+        self._setClient()
+
+        triggerPath = os.path.join(self.client_root, "job_trigger.py")
+        with open(triggerPath, "w") as f:
+            f.write(triggerCode)
+        self.p4.run_add(triggerPath)
+        self.p4.run_submit("-d","Added trigger")
+        
+        files = self.p4.run_files("//...")
+        self.assertEqual(len(files), 1, "Not exactly one file stored")
+        self.assertEqual(files[0]["depotFile"], "//depot/job_trigger.py", "File not found where expected")
+        
+        jobTrigger = 'jobtest form-out job ' \
+                     '"{0} %//depot/job_trigger.py% %specdef% %formname% %formfile%"'.format(sys.executable)
+        
+        triggers = self.p4.fetch_triggers()
+        triggers._triggers = [ jobTrigger ]
+        self.p4.save_triggers(triggers)
+        
+        # need to bounce connecting to reload trigger table
+        self.p4.disconnect()
+        self.p4.connect()
+                
+        jobspec = self.p4.fetch_jobspec()        
+        jobspec._fields.append("110 Project word 32 optional")
+        self.p4.save_jobspec(jobspec)
+        
+        job = self.p4.fetch_job("myjob")
+        
+        self.assertEqual(job._status, "suspended", "Trigger did not change status to suspended")
+        job._description = "Testing jobspec and job triggers"
+        job._project = "NewProject"
+        self.p4.save_job(job)
+        
+        job = self.p4.fetch_job("myjob")
+        self.assertEqual(job._job, "myjob", "Job name not correct")
+        self.assertEqual(job._project, "NewProject", "Job project name not set")
+
+    def testProtectionWithComment( self ):
+        # create protection table with comments
+        # save protection table
+        # reload protection table
+        # verify it works
+        
+        protectionView = ["## First line",
+                          "write user * * //... ## standard",
+                          "admin user tom 127.0.0.1 //... ## special admin user"
+                          "## Super entry following",
+                          "super user {0} * //... ## standard super user".format(self.p4.user)
+                         ]
+        
+        self.p4.connect()
+        current_pwd = self.server_root
+
+        if self.p4.server_level >= 41: # 2016.1
+            # double check we have the right patch level
+            if self.getServerPatchLevel(self.p4.run_info()) >= 1398982:
+                protect = self.p4.fetch_protect()
+                protect._protections = protectionView
+                self.p4.save_protect(protect)
+    
+                protect = self.p4.fetch_protect()
+                
+                self.assertEqual(protectionView, protect._protections, "Views are not identical")
+            else:
+                print("\n*** Please upgrade to at least 2016.1 Patch 2 (1398982) ***")
+    
 if __name__ == '__main__':
     unittest.main()
 
